////////////////////////////////////////////////////////////////////////////////////
// funcións de escritura e lectura en eeprom
////////////////////////////////////////////////////////////////////////////////////
// - read(), write() leen/escriben 1 byte
// - EEPROM.length() indica tamaño
// - get(), put() leen/escriben unha variable ou estructura completa pero non para
// cadenas. Hai que facelo byte a byte. Hai que calcular a lonxitude con strlen() 
// e engadir 0 final. Put() verifica datos e non escribe se son iguais
// - tampouco valen para IPAddress, é unha clase e non un array. Tamén byte a byte.
////////////////////////////////////////////////////////////////////////////////////
// función de escritura de configuración
// - cadena "CONF": ocupa os 4 primeiros bytes. Indica configuración válida
// - parámetro 1: número de medidor (1 byte)
// - parámetro 2: mac local (6 bytes)
// - parámetro 3: ip local (4 bytes)
// - parámetro 4: máscara local (4 bytes)
// - parámetro 5: gateway (4 bytes)
// - parámetro 6: dns (4 bytes)
// - parámetro 7: nome de host (lonx. strlen+1)
// - parámetro 8: nome de ruta (lonx. strlen+1) 

////////////////////////////////////////////////////////////////////////////////////

unsigned int eeprom_dir; //variable de posición actual de lectura ou escritura

////////////////////////////////////////////////////////////////////////////////////
// función que recupera os parámetros da eeprom
// devolve 0=erro, 1=ok
byte eeprom_lee_configuracion()
{
  byte res, dato;

  eeprom_dir=0; //inicio de memoria
  //verifica etiqueta inicial "CONF", se coincide lee valores
  res=1; 
  dato=EEPROM.read(eeprom_dir); if(dato!='C') res=0; eeprom_dir++;
  dato=EEPROM.read(eeprom_dir); if(dato!='O') res=0; eeprom_dir++;
  dato=EEPROM.read(eeprom_dir); if(dato!='N') res=0; eeprom_dir++;
  dato=EEPROM.read(eeprom_dir); if(dato!='F') res=0; eeprom_dir++;
  dato=EEPROM.read(eeprom_dir); if(dato!=0)   res=0; eeprom_dir++;

  if(res) //etiqueta ok, lee configuración de eeprom
  {
    medidor_numero=EEPROM.read(eeprom_dir); eeprom_dir++; //número de medidor
    EEPROM.get(eeprom_dir,local_mac);    eeprom_dir+=6;
    local_ip=eeprom_lee_ip();
    local_mask=eeprom_lee_ip();
    gw_ip=eeprom_lee_ip();
    dns_ip=eeprom_lee_ip();
    eeprom_lee_cadena(medidor_host);
    eeprom_lee_cadena(medidor_ruta);

//    Serial.println(F("Datos eeprom: OK"));
  }
//  else 
//    Serial.println(F("Datos eeprom: ERRO"));

  return res; //0=erro, 1=ok
}
////////////////////////////////////////////////////////////////////////////////////
// función que escribe os parámetros de configuración na memoria eeprom
void eeprom_escribe_configuracion()
{
  eeprom_dir=0;
  EEPROM.put(eeprom_dir,"CONF");  eeprom_dir+=5; //lonxitude de "CONF"+1 (0 final)
  EEPROM.write(eeprom_dir,medidor_numero); eeprom_dir++; //número de medidor
  EEPROM.put(eeprom_dir,local_mac); eeprom_dir+=6;
  eeprom_escribe_ip(local_ip);
  eeprom_escribe_ip(local_mask);
  eeprom_escribe_ip(gw_ip);
  eeprom_escribe_ip(dns_ip);
  eeprom_escribe_cadena(medidor_host);
  eeprom_escribe_cadena(medidor_ruta);

}
////////////////////////////////////////////////////////////////////////////////////
// escribe un enderezo ip a partir da posición actual da eeprom
void eeprom_escribe_ip(IPAddress ip)
{
  EEPROM.write(eeprom_dir,ip[0]); eeprom_dir++;  
  EEPROM.write(eeprom_dir,ip[1]); eeprom_dir++;  
  EEPROM.write(eeprom_dir,ip[2]); eeprom_dir++;  
  EEPROM.write(eeprom_dir,ip[3]); eeprom_dir++;  
}
////////////////////////////////////////////////////////////////////////////////////
// lee un enderezo ip a partir da posición actual da eeprom
IPAddress eeprom_lee_ip()
{
  IPAddress ip;
  ip[0]=EEPROM.read(eeprom_dir); eeprom_dir++;
  ip[1]=EEPROM.read(eeprom_dir); eeprom_dir++;
  ip[2]=EEPROM.read(eeprom_dir); eeprom_dir++;
  ip[3]=EEPROM.read(eeprom_dir); eeprom_dir++;
  return ip;
}

////////////////////////////////////////////////////////////////////////////////////
// escritura de cadena de texto en eeprom
// a cadena ten que ser curta e terminada en 0
void eeprom_escribe_cadena(char *cad)
{ //sempre escribe alomenos un dato (0 final)
  char dato=' ';
  while (dato!=0) 
  { dato=*cad; EEPROM.write(eeprom_dir,dato); 
    cad++; eeprom_dir++; } 
}

////////////////////////////////////////////////////////////////////////////////////
// lectura de cadena de texto de eeprom
// a cadena ten que ser curta e terminada en 0
void eeprom_lee_cadena(char *cad)
{ // sempre lee alomenos un dato (0=fin)
  char dato=' ';
  while (dato!=0)
  { dato=EEPROM.read(eeprom_dir); 
    *cad=dato; cad++; eeprom_dir++; } 
}
////////////////////////////////////////////////////////////////////////////////////

