////////////////////////////////////////////////////////////////////////////////////////////
// funcións de comunicacións cliente-servidor ip
////////////////////////////////////////////////////////////////////////////////////////////
// definicións para envío datos http
// exemplo de secuencia completa:
// "GET /meteoproval/datos.php?TX=1  HTTP/1.1"+CR+LF
// "Host: meteoproval.es"+CR+LF+CR+LF
// NOTA: en Arduino os saltos de liña fanse con println(), ver exemplo: "WebClientRepeating"
// a función println engade "\r\n" (0x13,0x10 = CR,LF) ó final dos datos enviados

//definicións de cadenas trasladadas a módulo html
//#define HTTP_GET F("GET /")
//#define HTTP_FINGET F(" HTTP/1.1") //engadir cr+lf ou println
//#define HTTP_HOST F("Host: ")    //engadir cr+lf ou println
//#define HTTP_CLOSE F("Connection: close") //engadir cr+lf ou println e liña baleira como final
//
// definicións de módulo ethernet
#define ETHERNET_PIN_RESET 9 //pin empregado para reset do módulo ethernet

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// baseado nos exemplos de código de Arduino: 
// https://www.arduino.cc/en/Tutorial/WebClientRepeating
//
/*
  Repeating Web client

 This sketch connects to a a web server and makes a request
 using a Wiznet Ethernet shield. You can use the Arduino Ethernet shield, or
 the Adafruit Ethernet shield, either one will work, as long as it's got
 a Wiznet Ethernet module on board.

 This example uses DNS, by assigning the Ethernet client with a MAC address,
 IP address, and DNS address.

 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13

 created 19 Apr 2012
 by Tom Igoe
 modified 21 Jan 2014
 by Federico Vanzati

 http://www.arduino.cc/en/Tutorial/WebClientRepeating
 This code is in the public domain.
 */

////////////////////////////////////////////////////////////////////////////////////////////
// función que recibe peticións de páxinas por parte de clientes
void http_entrada()
{
  unsigned char contador; //número de caracteres recibidos do cliente
  char dato;

  // se hai algún cliente conectado recibe datos
  cliente = server.available();
  // se hai un cliente conectado recibe a petición e devolve páxina
  if (cliente)
  {
    Serial.println(F("cliente conectado"));
    contador=0; //inicia número de caracteres recibidos
    // an http request ends with a blank line
    boolean currentLineIsBlank = true;
    while (cliente.connected()) {
      if (cliente.available()) {
        dato = cliente.read(); //recibe carácter
//        Serial.write(dato); //envía carácter a consola
        //garda os MAXBUF primeiros caracteres recibidos, descarta resto
        if(contador<MAXBUF)
        { linea[contador]=dato;
          contador++; } 
        
        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
        if (dato == '\n' && currentLineIsBlank) 
          break; //sae do bucle while
       
        if (dato == '\n') 
        { // you're starting a new line
          currentLineIsBlank = true;
        } else if (dato != '\r') {
          // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      } // fin de if cliente
    } // fin de while connected

    //-------------------------------------------------------------------
    // fin de petición do cliente
    // agora hai que procesar a cabeceira recibida e enviar a páxina html
    linea[MAXBUF]=0; //marca de fin de liña
    Serial.println(linea); //imprime liña para verificar a cabeceira
    
    //función que lee a cabeceira e procesa a información:
    http_procesa();

    // give the web browser time to receive the data
    delay(10);
    // close the connection:
    cliente.stop();
    Serial.println(F("cliente desconectado"));
  }
}
////////////////////////////////////////////////////////////////////////////////////////////
// función que lee a petición http e extrae os parámetros:
// - nome da páxina solicitada (ningún para a páxina principal)
// - variables devoltas polos formularios
// a información se recibe na cadena global 'linea' (máx MAXBUF caracteres, <255)
void http_procesa()
{
  int res;
  byte numpax;
  char *plin, *plin2;
  //a consulta sempre empeza con GET e a primeira liña remata con HTTP...
  //exemplo: "GET /meteoproval/datos.php?TX=1  HTTP/1.1"+CR+LF
  //para verificar a liña buscamos as dúas secuencias e se non están non segue:
  res=1; //resultado da verificación, se vale 0 remata
  plin=strstr(linea,"GET"); if(plin==NULL) res=0; //localiza cadena "GET"
  //se atopa "GET" busca "/" e garda o seguinte carácter como novo inicio
  if(res) { plin=strstr(linea,"/"); if(plin==NULL) res=0; else plin++; }   
  //a continuación busca "HTTP" e marca o fin de liña nese punto
  if(res) { plin2=strstr(plin,"HTTP"); if(plin2==NULL) res=0; }   
  if(res) { *plin2=0; }
  
  //a continuación rastrexa a cadena para localizar os nomes de páxinas ou variables
  if(res) 
  { numpax=0; //inicialmente asigna páxina nula
    // busca páxina index.htm (se a atopa tamén vale para index.html)
    plin2=strstr(plin,"index.htm"); if(plin2) numpax=1; //asigna número de páxina
    // busca páxina conf1.htm (presenta configuración do medidor)
    plin2=strstr(plin,"conf1.htm"); if(plin2) numpax=2; //asigna número de páxina 
    // busca páxina conf2.htm (modificación de configuración)
    plin2=strstr(plin,"conf2.htm"); if(plin2) numpax=3; //asigna número de páxina 
    // busca páxina conf3.htm (gardar novos parámetros)
    plin2=strstr(plin,"conf3.htm"); if(plin2) numpax=4; //asigna número de páxina 

    //unha vez rematada a busca envía a páxina indicada
    switch(numpax)
    {
      case 0:  //non hai nome de páxina, envía inicio
      case 1: sensores_lectura(); //fai unha nova lectura dos sensores
              paxina_html_inicio(); break; //envía páxina de inicio
      case 2: paxina_html_presenta_configuracion(1); break; //envía páxina de rede só lectura
      case 3: paxina_html_presenta_configuracion(0); break; //envía páxina de rede formulario
      case 4: paxina_html_cambia_configuracion(plin); break; //envía confirmación ip
    } //fin de switch
  } // fin de busca de nomes
  
}
////////////////////////////////////////////////////////////////////////////////////////////
// función de configuración do módulo ethernet
// - a función begin() non devolve nada en configuración manual (sen dhcp)
// - mellor evitar dhcp, usa moitos recursos do sistema
// función de inicio completa: Ethernet.begin(mac, ip, dns, gateway, subnet); 
void http_inicio() 
{
  byte i;
  
  Serial.println(F("INICIO REDE"));

  // pecha sempre a conexión anterior se existía
  cliente.stop();

  // activa reset no módulo ethernet
  pinMode(ETHERNET_PIN_RESET,OUTPUT);
  digitalWrite(ETHERNET_PIN_RESET,LOW); //activa reset (a 1)
  delay(1000); //espera 1 segundo
  digitalWrite(ETHERNET_PIN_RESET,HIGH); //desactiva reset

  // espera un tempo a que se inicie o módulo
  delay(5000);
  // start the Ethernet connection using a fixed IP address and DNS server:
  Ethernet.begin(local_mac, local_ip, dns_ip, gw_ip, local_mask);

  // presenta os datos actuais de conexión
  Serial.print(F(" Hardware: ")); 
  switch(Ethernet.hardwareStatus())
  {  case EthernetNoHardware: Serial.println(F("Modulo non existe")); break;
     case EthernetW5100:      Serial.println(F("Wiznet 5100"  )); break;
     case EthernetW5200:      Serial.println(F("Wiznet 5200"  )); break;
     case EthernetW5500:      Serial.println(F("Wiznet 5500"  )); break;  }

  Serial.print(F(" Conexion: ")); 
  switch(Ethernet.linkStatus())
  {  case Unknown:  Serial.println(F("indeterminada")); break;
     case LinkON:   Serial.println(F("activa"));          break;
     case LinkOFF:  Serial.println(F("inactiva"));        break;  }

// imprime MAC, IP, máscara, gw, dns, nombre de host e ruta de script
  Serial.print(F(" MAC: ")); for (i=0;i<6;i++) Serial.print(local_mac[i],HEX);

  Serial.print(F(" IP local: ")); Serial.print(Ethernet.localIP());
//  Serial.print(F(" MASK local: ")); Serial.print(local_mask);
  Serial.print(F(" MASK local: ")); Serial.print(Ethernet.subnetMask());
//  Serial.print(F(" GW: ")); Serial.print(gw_ip);
  Serial.print(F(" GW: ")); Serial.print(Ethernet.gatewayIP());
//  Serial.print(F(" DNS: ")); Serial.print(dns_ip);
  Serial.print(F(" DNS: ")); Serial.print(Ethernet.dnsServerIP());

  Serial.print(F(" HOST: ")); Serial.print(medidor_host);
  Serial.print(F(" RUTA: ")); Serial.print(medidor_ruta);

  Serial.println();

  // start the server
  server.begin();
//  Serial.print(F("IP do servidor: ")); Serial.println(Ethernet.localIP());
  
}

//////////////////////////////////////////////////////////////////////////////////
// función que intenta a conexión co servidor http e envía a consulta con datos
// - se o parámetro vale 1 manda unha trama cos valores da matriz radio_datos[]
// - se o parámetro vale 0 manda unha trama nula (datos=0 e suma=0xffff)
void http_consulta(char param)
{
  // pecha sempre a conexión anterior se existía
  cliente.stop();

  //Conexión por nome, se funciona envía consulta
  //(elimino conexión por ip por simplicidade)
  //NOTA: verificar que devolve SUCCESS, se non pode haber erro e pensar que está ben
  if (cliente.connect(medidor_host, MEDIDOR_PORT)==1) //conexión correcta 
    http_envia_datos(param); 
  else //fallo de conexión
  {
    Serial.println(F("fallo conexion http"));
    http_inicio(); //intenta reiniciar a rede
  }

}
//////////////////////////////////////////////////////////////////////////////////

void http_envia_datos(char param)
{
  unsigned char i; 
  unsigned int suma; 

  Serial.print(F("conexion a servidor "));Serial.println(medidor_host);

/*// exemplo: 
    cliente.println("GET /tempo.php HTTP/1.1");
    cliente.println("Host: codos.meteoproval.es");
    cliente.println("Connection: close");
    cliente.println();
//*/

  // envía consulta http con datos:
  html(GET);
  // crea cadea de variables para script php
  // usa sprintf() con formato '%05u' = unsigned int con 5 dixitos e 0 iniciais
  // primeiro envía o nome do script ('datos.php' ou similar) con ruta completa
  cliente.print(medidor_ruta); Serial.print(medidor_ruta);
  // identificador de medidor (2 díxitos)
  sprintf(cad1,"?MED=%02u",medidor_numero);cliente.print(cad1);Serial.print(cad1); 
  // 10 variables consecutivas (nomes 'A'..'J')
  if(param==1) //trama válida
  { suma=0;
    for (i=0;i<10;i++)
    { sprintf(cad1,"&%c=%05u",i+'A',datos[i]);cliente.print(cad1); suma+=datos[i]; Serial.print(cad1);}
    // por último imprime suma de control (variable 'S')
    sprintf(cad1,"&S=%05u",suma);cliente.print(cad1); Serial.print(cad1); }
  else //trama nula
    cliente.print(F("&A=0&B=0&C=0&D=0&E=0&F=0&G=0&H=0&I=0&J=0&S=99")); //erro de suma     

  //envía ip local do medidor para conexión a microservidor (resto de parámetros se supoñen coñecidos)
  sprintf(cad1,"&IP=%02X%02X%02X%02X",local_ip[0],local_ip[1],local_ip[2],local_ip[3]);cliente.print(cad1);Serial.print(cad1);
  
  html(FINGET);
  html(HOST); cliente.println(medidor_host);
  html(CONNCLOSE);


  //espera resposta e imprime se hai 
  for (i=0;i<100;i++)
  { //espera resposta ata un tempo máximo de 1s
    delay(10); //ou mentres cheguen datos
    while(cliente.available()) 
    { char c = cliente.read();
      Serial.write(c); }
  }

  // unha vez recibida a resposta pecha a conexión
  cliente.stop();
    
}
//////////////////////////////////////////////////////////////////////////////////


